#include "timelinemonthmodel.h"
#include <QVariant>
#include <algorithm>

using namespace Qt::Literals::StringLiterals;

namespace {

void advanceMonth(QCalendar::YearMonthDay &ymd)
{
    if (ymd.month == 12) {
        ymd.month = 1;
        ymd.year++;
    } else {
        ymd.month++;
    }
}

} // namespace

TimelineMonthModel::TimelineMonthModel(QObject *parent)
    : QAbstractListModel(parent)
    , m_fromDate()
    , m_toDate()
    , m_locale()
    , m_months()
{}

int TimelineMonthModel::rowCount(const QModelIndex &parent) const
{
    if (parent.isValid()) {
        return 0;
    }
    return m_months.length();
}

QVariant TimelineMonthModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid() || index.row() >= rowCount()) {
        return {};
    }

    Month month = m_months.at(index.row());
    switch (role) {
    case Qt::DisplayRole:
        return m_locale.standaloneMonthName(month.number);
    case DayCount:
        return month.days;
    }

    return {};
}

QHash<int, QByteArray> TimelineMonthModel::roleNames() const
{
    QHash<int, QByteArray> roles = QAbstractListModel::roleNames();
    roles[DayCount] = "dayCount"_ba;
    return roles;
}

QDate TimelineMonthModel::fromDate() const
{
    return m_fromDate;
}

void TimelineMonthModel::setFromDate(QDate date)
{
    if (date == m_fromDate) {
        return;
    }
    m_fromDate = date;
    emit fromDateChanged(m_fromDate);
    updateMonths();
}

QDate TimelineMonthModel::toDate() const
{
    return m_toDate;
}

void TimelineMonthModel::setToDate(QDate date)
{
    if (date == m_toDate) {
        return;
    }
    m_toDate = date;
    emit toDateChanged(m_toDate);
    updateMonths();
}

QLocale TimelineMonthModel::locale() const
{
    return m_locale;
}

void TimelineMonthModel::setLocale(const QLocale &locale)
{
    if (locale == m_locale) {
        return;
    }
    m_locale = locale;
    emit localeChanged(m_locale);
    if (m_months.isEmpty()) {
        return;
    }
    emit dataChanged(index(0), index(rowCount() - 1), {Qt::DisplayRole});
}

void TimelineMonthModel::updateMonths()
{
    QCalendar gregorian;
    QCalendar::YearMonthDay fromYMD = gregorian.partsFromDate(std::min(m_fromDate, m_toDate));
    QCalendar::YearMonthDay toYMD = gregorian.partsFromDate(std::max(m_fromDate, m_toDate));

    if (!fromYMD.isValid() || !toYMD.isValid()) {
        if (m_months.isEmpty()) {
            return;
        }
        beginResetModel();
        m_months.clear();
        endResetModel();
        return;
    }

    beginResetModel();
    m_months.clear();
    if (fromYMD.year == toYMD.year && fromYMD.month == toYMD.month) {
        m_months.push_back({fromYMD.month, toYMD.day - fromYMD.day + 1});
    } else {
        m_months.push_back(
            {fromYMD.month, gregorian.daysInMonth(fromYMD.month, fromYMD.year) - fromYMD.day + 1});
        advanceMonth(fromYMD);
        for (; fromYMD.month != toYMD.month || fromYMD.year != toYMD.year; advanceMonth(fromYMD)) {
            m_months.push_back({fromYMD.month, gregorian.daysInMonth(fromYMD.month, fromYMD.year)});
        }
        m_months.push_back({toYMD.month, toYMD.day});
    }
    endResetModel();
}

#include "moc_timelinemonthmodel.cpp"
